classdef rDataBase < handle
    %RDATABASE Data base of CARRE Project repository.
    %   In future all queries should be done via this objectt.
    
    properties
        RiskFactors;
        RiskEvidences;
        RiskElements;
        Observables;
        Terms;
        Connection;
    end
    
    methods
        
        function obj = rDataBase(varargin)
            obj.RiskFactors = {};
            obj.RiskEvidences = {};
            obj.RiskElements = {};
            obj.Observables = {};
            obj.Terms = {};
            
            if length(varargin) == 1
                %connection structure passed
                obj.Connection = varargin{1};
            elseif length(varargin) == 2
                %create new connection
                obj.Connection = rConnect( varargin{1}, varargin{2});
            else
                %no connection structure
                obj.Connection = [];
            end
        end
        
        function obj = updateRiskFactors(obj)
            obj.RiskFactors = rGetRiskFactors( obj.Connection );
        end
        
        function obj = updateRiskEvidences(obj)
            obj.RiskEvidences = rGetRiskEvidences( obj.Connection );
        end
        
        function obj = updateRiskElements(obj)
            obj.RiskElements = rGetRiskElements( obj.Connection );
        end
        
        function obj = updateObservables(obj)
            obj.Observables = rGetObservables( obj.Connection );
        end
        
        function obj = updateTerms(obj)
            obj.Terms = rGetTerms( obj.Connection );
        end
        
        function obj = updateAll(obj)
            disp('updateRiskFactors');
            obj = updateRiskFactors(obj);
            disp('updateRiskEvidences');
            obj = updateRiskEvidences(obj);
            disp('updateRiskElements');
            obj = updateRiskElements(obj);
            disp('updateObservables');
            obj = updateObservables(obj);
            disp('updateTerms');
            obj = updateTerms(obj);
        end
        
        function saveDataBase(obj, filename)
            RiskFactors = obj.RiskFactors;
            RiskEvidences = obj.RiskEvidences;
            RiskElements = obj.RiskElements;
            Observables = obj.Observables;
            Terms = obj.Terms;
            save(filename, 'RiskFactors', 'RiskEvidences', 'RiskElements', 'Observables', 'Terms');
        end
        
        function loadDataBase(obj, filename)
            s = load(filename);
            obj.RiskFactors = s.RiskFactors;
            obj.RiskEvidences = s.RiskEvidences;
            obj.RiskElements = s.RiskElements;
            obj.Observables = s.Observables;
            obj.Terms = s.Terms;
        end
        
        function nodes = searchObservableByName(obj, obsvName, mode)
            %mode:
            % 1 (default) - find, case insersivity
            % 2 - find, case sensivity
            % 3 - exact
            if nargin <= 2
                mode = 1;
            end
            nodes = [];
            switch mode
                case 1
                    nodes = {obj.Observables(~cellfun(@isempty, strfind(lower({obj.Observables.name})', lower(obsvName)))).node}';
                case 2
                    nodes = {obj.Observables(~cellfun(@isempty, strfind({obj.Observables.name}', obsvName))).node}';
                case 3
                    nodes = {obj.Observables(strcmp({obj.Observables.name}', obsvName)).node};
            end
            
            if length(nodes) == 1
                nodes = cell2mat(nodes);
            end
        end
        
        function obsv = getObservableByNode(obj, node)
            obsv = obj.Observables(strcmp({obj.Observables.node}', node));
        end
        
        function riskEl = getRiskElementByNode(obj, node)
            riskEl = obj.RiskElements(strcmp({obj.RiskElements.node}', node));
        end
        
        function riskF = getRiskFactorByNode(obj, node)
            riskF = obj.RiskFactors(strcmp({obj.RiskFactors.node}', node));
        end
        
        function riskEv = getRiskEvidenceByNode(obj, node)
            riskEv = obj.RiskEvidences(strcmp({obj.RiskEvidences.node}', node));
        end
        
    end
    
end

