function [ riskEv ] = rGetRiskEvidences( con )

%some constants
expression = 'http://carre.kmi.open.ac.uk/ontology/risk.owl#has_observable_expression';
ratio = 'http://carre.kmi.open.ac.uk/ontology/risk.owl#has_risk_evidence_ratio_value';
ratioMax = 'http://carre.kmi.open.ac.uk/ontology/risk.owl#risk_evidence_maximum_confidence_interval';
ratioMin = 'http://carre.kmi.open.ac.uk/ontology/risk.owl#risk_evidence_minimum_confidence_interval';
hasObsv = 'http://carre.kmi.open.ac.uk/ontology/risk.owl#has_risk_evidence_observable';
radioType = 'http://carre.kmi.open.ac.uk/ontology/risk.owl#has_risk_evidence_ratio_type';
adjustedFor = 'http://carre.kmi.open.ac.uk/ontology/risk.owl#is_adjusted_for';
source = 'http://carre.kmi.open.ac.uk/ontology/risk.owl#has_risk_evidence_source';

% getting observables nodes from server
[ nodes ] = rGetNodesByType( con, 'r:risk_evidence' );
% we are interested in nodes that are named 'node'
nodes = nodes(~cellfun(@isempty,strfind(nodes, 'node')));

%query each observables for atributes
riskEv = struct([]);
h = waitbar(0,'Wait');
for k = 1:length(nodes)
    waitbar(k/length(nodes), h, sprintf('Wait %d/%d', k, length(nodes)));
    [ atr ] = rGetNodeAtributes( con, nodes{k} );
    if ~isempty(atr)
        riskEv(end+1).plainTextAtributes = atr;
        riskEv(end).node = nodes{k};
        
        %getting expression
        ind = find(strcmp(atr(:,1), expression), 1);
        if ~isempty(ind)
            riskEv(end).expression = atr{ind,2};
        end
        
        %getting source
        ind = find(strcmp(atr(:,1), source), 1);
        if ~isempty(ind)
            riskEv(end).source = atr{ind,2};
        end

        %getting ratioMin
        ind = find(strcmp(atr(:,1), ratioMin), 1);
        if ~isempty(ind)
            riskEv(end).ratio(1) = str2double(atr{ind,2});
        end
        
        %getting ratio
        ind = find(strcmp(atr(:,1), ratio), 1);
        if ~isempty(ind)
            riskEv(end).ratio(2) = str2double(atr{ind,2});
        end
        
        %getting ratioMax
        ind = find(strcmp(atr(:,1), ratioMax), 1);
        if ~isempty(ind)
            riskEv(end).ratio(3) = str2double(atr{ind,2});
        end
        
        %getting OBSV
        ind = find(strcmp(atr(:,1), hasObsv));
        if ~isempty(ind)
            riskEv(end).obsv = atr(ind,2);
        end
        
        %getting radioType
        ind = find(strcmp(atr(:,1), radioType), 1);
        if ~isempty(ind)
            S = atr{ind,2};
            from = strfind(S, '#');
            riskEv(end).radioType = S(from+1:end);
        end
        
        %getting adjustedFor
        ind = find(strcmp(atr(:,1), adjustedFor));
        if ~isempty(ind)
            riskEv(end).adjustedFor = atr(ind,2);
        end
        
    end
end
close(h);
%sorting
[~, ind]=sort({riskEv.node});
riskEv = riskEv(ind);

end