classdef rPatient < handle
    %RPATIENT Patient object
    
    properties
        db;
        associatedObservables;
        riskElements;
        riskFactors;
    end
    
    methods
        function obj = rPatient(database)
            obj.db = database;
            obj.associatedObservables = struct('node', {}, 'val', {});
            
            %type - 'noObsv', 'missingObsv', 'fullObsv'
            %val - if empty -> lack of answer or missing data
            obj.riskElements = struct('riskElNode', {}, 'knownObsv', {}, 'unknownObsv', {}, 'type', {}, 'val', {});
            
        end
        
        function obj = setObservable(obj, obsvNode, value)
            %first check existence of observable in local associated obsv
            ind = find(strcmp({obj.associatedObservables.node}', obsvNode), 1);
            if isempty(ind)
                obj.associatedObservables(end+1).node = obsvNode;
                obj.associatedObservables(end).val = {value};
            else
                obj.associatedObservables(ind).node = obsvNode;
                obj.associatedObservables(ind).val = {value};
            end
        end
        
        function obj = deleteObservable(obj, obsvNode)
            ind = find(strcmp({obj.associatedObservables.node}', obsvNode), 1);
            if ~isempty(ind)
                obj.associatedObservables(ind) = [];
            end
        end
        
        function printRaport(obj, mode)
            % mode = {'full', 'reduced'};
            if nargin == 1
                mode = 'reduced';
            end
            if ~isempty(obj.associatedObservables)
                fprintf('\n--- Associated observables --- \n\n');
                for k = 1:length(obj.associatedObservables)
                    n = obj.db.getObservableByNode(obj.associatedObservables(k).node);
                    name = n.name;
                    val = obj.associatedObservables(k).val{1};
                    if ~ischar(val)
                        val = num2str(val);
                    end
                    fprintf('%d: %s = %s\n', k, name, val);
                end
            end
            
            if ~isempty(obj.riskElements)
                fprintf('\n--- Risk elements raport --- \n\n');
                for k = 1:length(obj.riskElements)
                    
                    [type] = obj.statusOfRiskElement(obj.riskElements(k).riskElNode);
                    if strcmp(type, 'noObsv') && ~strcmp(mode, 'full')
                        continue;
                    end
                	obj.printRiskElement(obj.riskElements(k).riskElNode);
                        fprintf('\n');
                end
            end
            
            if ~isempty(obj.riskFactors)
                fprintf('--- Risk factors raport --- \n\n');
                for k = 1:length(obj.riskFactors)
                	obj.printRiskFactor(obj.riskFactors(k).rfNode);
                end
            end
            
        end
        
        function printRiskElement(obj, node)
                    rInd = find(strcmp({obj.riskElements(:).riskElNode}, node));
                    rElement = obj.riskElements(rInd);
                    if isempty(rElement)
                        return;
                    end
                    r = obj.db.getRiskElementByNode(rElement.riskElNode);
                    rName = r.name;
                    rCondition = r.condition;
                    if isempty(rCondition)
                        rCondition = 'NONE';
                    end
                    rCondition = strrep(rCondition,sprintf('\n'),' ');
                    val = rElement.val;
                    if isempty(val)
                        val = 'NONE';
                    end
                    
                    [type, revelant] = obj.statusOfRiskElement(rElement.riskElNode);
                    
                    fprintf('Risk element nr %d: %s \n  val:\t\t\t%s\n  condition:\t%s\n  type:\t\t\t%s\n  revelant: \t%s\n\n', rInd, rName, val, rCondition, type, num2str(revelant));
                    
                    if ~isempty(rElement.Obsv)
                        disp('  Last observables state:');
                        for k2 = 1:size(rElement.Obsv, 1)
                            obs = obj.db.getObservableByNode(rElement.Obsv{k2,1});
                            oName = obs.name;
                            oVal = rElement.Obsv{k2,2};
                            if isempty(oVal)
                                oVal = 'UNKNOWN';
                            elseif ~ischar(oVal)
                                oVal = num2str(oVal);
                            end
                            
                            if revelant(k2)
                                revStr = 'up to date';
                            else
                                revStr = 'out of date';
                            end
                            
                            fprintf('    %d: %s = %s (%s)\n', k2, oName, oVal, revStr);
                        end
                    end
        end
        
        function obj = processRiskFactors(obj)
            obj.riskFactors = struct('rfNode', {}, 'rElements', {}, 'rEvidences', {}, 'outputRatio', {});
            for k = 1:length(obj.db.RiskFactors)
                % check risk elements
                condition = true;
                rElements = {};
                for k2 = 1:length(obj.db.RiskFactors(k).source)
                    ind = find(strcmp({obj.riskElements(:).riskElNode}, obj.db.RiskFactors(k).source{k2}), 1);
                    if isempty(ind) || strcmp(obj.riskElements(ind).val, '_noanswer') || strcmp(obj.riskElements(ind).val, '_noInfo') 
                        condition = false;
                    end
                    rElements{k2} = obj.riskElements(ind).riskElNode;
                end
                
                if condition
                    % add risk factor to local db
                    obj.riskFactors(end+1).rfNode = obj.db.RiskFactors(k).node;
                    obj.riskFactors(end).rElements = rElements;
                    obj.riskFactors(end).outputRatio = NaN;
                    
                    obj.riskFactors(end).rEvidences = struct('rEvNode', {}, 'observables', {}, 'type', {}, 'val', {});
                    for k2 = 1:length(obj.db.RiskFactors(k).evidence)
                        rEvNode = obj.db.RiskFactors(k).evidence(k2);
                        obj.riskFactors(end).rEvidences(k2).rEvNode = rEvNode;
                        obj.riskFactors(end).rEvidences(k2).val = NaN;
                        rEv = obj.db.getRiskEvidenceByNode(rEvNode);%obj.db.RiskEvidences(strcmp({obj.db.RiskEvidences(:).node}, rEvNode));
                        rEvObs = {};
                        for k3 = 1:length(rEv.obsv)
                            ind = find(strcmp({obj.associatedObservables(:).node}, rEv.obsv(k3)));
                            if any(ind)
                                rEvObs{k3,1} = rEv.obsv{k3};
                                rEvObs{k3,2} = obj.associatedObservables(ind).val{1};
                            else
                                rEvObs{k3,1} = rEv.obsv{k3};
                                rEvObs{k3,2} = [];
                            end
                        end
                        obj.riskFactors(end).rEvidences(k2).observables = rEvObs;
                        [type] = statusOfRiskEvidence(obj, obj.riskFactors(end).rEvidences(k2));
                        obj.riskFactors(end).rEvidences(k2).type = type;
                    end
                    
                    
                end
                
            end
            
        end
        
        function printRiskFactor(obj, node)
            rfInd = find(strcmp({obj.riskFactors(:).rfNode}, node));
            rFacL = obj.riskFactors(rfInd);
            if isempty(rFacL)
                return;
            end
            rFacDB = obj.db.getRiskFactorByNode(rFacL.rfNode);
            rName = rFacDB.name;
            rVal = rFacL.outputRatio;
            rTargetName = obj.db.getRiskElementByNode(rFacDB.target).name;
            rSourceNamesAndVals = {};
            for k = 1:length(rFacL.rElements)
                rSourceNamesAndVals{k,1} = obj.db.getRiskElementByNode(rFacL.rElements{k}).name;
                rSourceNamesAndVals{k,2} = obj.riskElements(strcmp({obj.riskElements(:).riskElNode}, rFacL.rElements{k})).val;
            end
            fprintf('Risk element nr %d: %s\n', rfInd, rName);
            fprintf('  Target:\t%s\n', rTargetName);
            fprintf('  Ratio:\t%.2f\n', rVal);
            fprintf('  Sources:\t\n');
            for k = 1:size(rSourceNamesAndVals,1)
                fprintf('    %d: %s = %s\n', k, rSourceNamesAndVals{k,1}, rSourceNamesAndVals{k,2});
            end
            
            fprintf('\n');
            for k = 1:length(rFacL.rEvidences)
                obj.printRiskEvidence(rFacL.rEvidences(k), k);
                fprintf('\n');
            end
        end
        
        function printRiskEvidence(obj, rEvidence, num)
            
            if nargin == 3
                numStr = sprintf(' nr %d', num);
            else
                numStr = '';
            end
            
            rEvNode = rEvidence.rEvNode;
            rEv = obj.db.getRiskEvidenceByNode(rEvNode);
            [type, revelant] = statusOfRiskEvidence(obj, rEvidence);
            
            expression = rEv.expression;
            if isempty(expression)
                expression = 'NONE';
            end
            expression = strrep(expression,sprintf('\n'),' ');
            
            val = rEvidence.val;
            
            fprintf('  Risk evidence%s:\n    type:\t\t\t%s\n    risk type:\t\t%s\n    risk ratio:\t\t[%.2f,%.2f,%.2f]\n    expression:\t\t%s\n    val:\t\t\t%d\n', ...
                numStr, type, rEv.radioType, rEv.ratio(1), rEv.ratio(2), rEv.ratio(3), expression, val);
            if ~isempty(rEvidence.observables)
                disp('    Last observables state:');
                for k2 = 1:size(rEvidence.observables, 1)
                    obs = obj.db.getObservableByNode(rEvidence.observables{k2,1});
                    oName = obs.name;
                    oVal = rEvidence.observables{k2,2};
                    if isempty(oVal)
                        oVal = 'UNKNOWN';
                    elseif ~ischar(oVal)
                        oVal = num2str(oVal);
                    end

                    if revelant(k2)
                        revStr = 'up to date';
                    else
                        revStr = 'out of date';
                    end

                    fprintf('      %d: %s = %s (%s)\n', k2, oName, oVal, revStr);
                end
            end
            
        end
        
        function [type, revelant] = statusOfRiskEvidence(obj, rEvidence)
            emptyObsv = 0;
            numOfObsv = size(rEvidence.observables,1);
            revelant = [];
            for k = 1:numOfObsv
                
                oVal = obj.associatedObservables(strcmp({obj.associatedObservables(:).node}, rEvidence.observables{k,1}));
                
                if ~isempty(oVal)
                    oVal = oVal.val{1};
                end
                
                if isempty(rEvidence.observables{k,2})
                    emptyObsv = emptyObsv+1;
                    revelant(k) = isempty(oVal);
                else
                    if isempty(oVal)
                        revelant(k) = false;
                    elseif ischar(oVal)
                        revelant(k) = strcmp(oVal, rEvidence.observables{k,2});
                    else
                        revelant(k) = (oVal == rEvidence.observables{k,2});
                    end
                end
            end
            
            if emptyObsv == 0
                type = 'fullObsv';
            elseif emptyObsv == numOfObsv
                type = 'noObsv';
            else
                type = 'partObsv';
            end
            
        end
        
        function obj = processRiskElements(obj)
            % clear data - for temporary propose
            obj.riskElements = struct('riskElNode', {}, 'Obsv', {}, 'val', {});
            for k = 1 : length(obj.db.RiskElements)
                obj.riskElements(k).riskElNode = obj.db.RiskElements(k).node;
                for k2 = 1:length(obj.db.RiskElements(k).obsv)
                    if any(strcmp({obj.associatedObservables(:).node}, obj.db.RiskElements(k).obsv(k2)))
                        obj.riskElements(k).Obsv{end+1,1} = obj.db.RiskElements(k).obsv{k2};
                        oVal = obj.associatedObservables(strcmp({obj.associatedObservables(:).node}, obj.riskElements(k).Obsv{end,1})).val{1};
                        obj.riskElements(k).Obsv{end  ,2} = oVal;
                    else
                        obj.riskElements(k).Obsv{end+1,1} = obj.db.RiskElements(k).obsv{k2};
                        obj.riskElements(k).Obsv{end  ,2} = [];
                    end
                end
                obj.riskElements(k).val = '_noanswer';
            end
        end
        
        function answerQuestions(obj)
            % process Risk Elements
            obj.updateRiskElements;
            fprintf('Answer questions.\nType nothing if some information to answer the question are not provided.\nType ''exit'' to stop answering questions and return to command line\n');
            
            for k = 1:length(obj.riskElements)
                [type, revelant] = obj.statusOfRiskElement(obj.riskElements(k).riskElNode);
                if strcmp(type, 'noObsv')
                    continue;
                end
                
                if ~strcmp(obj.riskElements(k).val, '_noanswer') && all(revelant)
                    continue;
                end
                    
                fprintf('\n');
                obj.printRiskElement(obj.riskElements(k).riskElNode);
                fprintf('\n');
                an = input('Type value of risk element expression: ', 's');
                if isempty(an)
                    if strcmp(obj.riskElements(k).val, '_noanswer')
                        obj.riskElements(k).val = '_noInfo';
                    end
                elseif strcmpi(an, 'exit')
                    return;
                else
                    obj.riskElements(k).val = an;
                end
            end
            
            %process risk factors
            obj.updateRiskFactors;
            for k1 = 1:length(obj.riskFactors)
                for k2 = 1:length(obj.riskFactors(k1).rEvidences)
                    if isnan(obj.riskFactors(k1).rEvidences(k2).val)
                        obj.printRiskEvidence(obj.riskFactors(k1).rEvidences(k2));
                        an = input('Type value of risk factor (0,1,NaN): ');
                        obj.riskFactors(k1).rEvidences(k2).val = an;
                        fprintf('\n\n');
                    end
                end
            end
            obj.updateRiskFactors;
        end
        
        function [type, revelant] = statusOfRiskElement(obj, node)
            rElement = obj.riskElements(strcmp({obj.riskElements(:).riskElNode}, node));
            revelant = [];
            emptyObsv = 0;
            numOfObsv = size(rElement.Obsv,1);
            for k = 1:numOfObsv
                
                oVal = obj.associatedObservables(strcmp({obj.associatedObservables(:).node}, rElement.Obsv{k,1}));
                
                if ~isempty(oVal)
                    oVal = oVal.val{1};
                end
                
                if isempty(rElement.Obsv{k,2})
                    emptyObsv = emptyObsv+1;
                    revelant(k) = isempty(oVal);
                else
                    if isempty(oVal)
                        revelant(k) = false;
                    elseif ischar(oVal)
                        revelant(k) = strcmp(oVal, rElement.Obsv{k,2});
                    else
                        revelant(k) = (oVal == rElement.Obsv{k,2});
                    end
                end
            end
            
            if emptyObsv == 0
                type = 'fullObsv';
            elseif emptyObsv == numOfObsv
                type = 'noObsv';
            else
                type = 'partObsv';
            end
            
        end
        
        function updateRiskElements(obj)
            %check status of revelacne of assiosiated observables and those
            %in risk elements
            
            for k = 1:length(obj.riskElements)
                [~, revelant] = obj.statusOfRiskElement(obj.riskElements(k).riskElNode);
                if ~any(revelant)
                    obj.riskElements(k).val = '_noanswer';
                    % update assiociated observables values
                    for k2 = 1:size(obj.riskElements(k).Obsv,1)
                        obj.riskElements(k).Obsv{k2,2} = obj.associatedObservables(strcmp({obj.associatedObservables.node}, obj.riskElements(k).Obsv{k2,1})).val{1};
                    end
                end
            end
        end
        
        function updateRiskFactors(obj)
            for k1 = 1:length(obj.riskFactors)
                obj.riskFactors(k1).outputRatio = NaN;
                for k2 = 1:length(obj.riskFactors(k1).rEvidences)
                [~, revelant] = obj.statusOfRiskEvidence(obj.riskFactors(k1).rEvidences(k2));
                    if ~isempty(revelant) && ~any(revelant)
                        obj.riskFactors(k1).rEvidences(k2).val = NaN;
                        % update assiociated observables values
                        for k3 = 1:size(obj.riskFactors(k1).rEvidences(k2).observables,1)
                            obj.riskFactors(k1).rEvidences(k2).observables{k3,2} = obj.associatedObservables(strcmp({obj.associatedObservables.node}, obj.riskFactors(k1).rEvidences(k2).observables{k3,1})).val{1};
                        end
                    end
                    if ~isnan(obj.riskFactors(k1).rEvidences(k2).val) && (obj.riskFactors(k1).rEvidences(k2).val == 1)
                        ratio = obj.db.getRiskEvidenceByNode(obj.riskFactors(k1).rEvidences(k2).rEvNode).ratio(2);
                        if isnan(obj.riskFactors(k1).outputRatio)
                            obj.riskFactors(k1).outputRatio = ratio;
                        else
                            obj.riskFactors(k1).outputRatio = max(ratio, obj.riskFactors(k1).outputRatio);
                        end
                    end
                end
            end
        end
        
        function rebuild(obj)
            % reprocess all data, answer questions etc
            obj.processRiskElements;
            obj.answerQuestions;
            obj.processRiskFactors;
            obj.answerQuestions;
        end
        
        function savePatient(obj, file)
            associatedObservables = obj.associatedObservables;
            riskElements = obj.riskElements;
            riskFactors = obj.riskFactors;
            save(file, 'associatedObservables', 'riskElements', 'riskFactors');
        end
        
        function loadPatient(obj, file)
            s = load(file);
            obj.associatedObservables = s.associatedObservables;
            obj.riskElements = s.riskElements;
            obj.riskFactors = s.riskFactors;
        end
        
        function riskFactorRaport(obj)
            for k = 1:length(obj.riskFactors)
                if ~isnan(obj.riskFactors(k).outputRatio)
                    rFacDB = obj.db.getRiskFactorByNode(obj.riskFactors(k).rfNode);
                    rVal = obj.riskFactors(k).outputRatio;
                    rTargetName = obj.db.getRiskElementByNode(rFacDB.target).name;
                    fprintf('Possibility of %s with risk ratio %.2f\n', rTargetName, rVal);
                end
            end
        end
        
    end
    
end

