function [ riskFac ] = rGetRiskFactors( con )

%some constants
source = 'http://carre.kmi.open.ac.uk/ontology/risk.owl#has_risk_factor_source';
target = 'http://carre.kmi.open.ac.uk/ontology/risk.owl#has_risk_factor_target';
evidence = 'http://carre.kmi.open.ac.uk/ontology/risk.owl#has_risk_evidence';
type = 'http://carre.kmi.open.ac.uk/ontology/risk.owl#has_risk_factor_association_type';
name = 'http://carre.kmi.open.ac.uk/ontology/risk.owl#risk_factor_title';

% getting observables nodes from server
[ nodes ] = rGetNodesByType( con, 'r:risk_factor' );
% we are interested in nodes that are named 'node'
nodes = nodes(~cellfun(@isempty,strfind(nodes, 'node')));

%query each observables for atributes
riskFac = struct([]);
h = waitbar(0,'Wait');
for k = 1:length(nodes)
    waitbar(k/length(nodes), h, sprintf('Wait %d/%d', k, length(nodes)));
    [ atr ] = rGetNodeAtributes( con, nodes{k} );
    if ~isempty(atr)
        riskFac(end+1).plainTextAtributes = atr;
        riskFac(end).node = nodes{k};
        
        %getting source
        ind = find(strcmp(atr(:,1), source));
        if ~isempty(ind)
            riskFac(end).source = atr(ind,2);
        end
        
        %getting target
        ind = find(strcmp(atr(:,1), target), 1);
        if ~isempty(ind)
            riskFac(end).target = atr{ind,2};
        end
        
        %getting evidence
        ind = find(strcmp(atr(:,1), evidence));
        if ~isempty(ind)
            riskFac(end).evidence = atr(ind,2);
        end
        
        %getting type
        ind = find(strcmp(atr(:,1), type), 1);
        if ~isempty(ind)
            S = atr{ind,2};
            from = strfind(S, '#');
            riskFac(end).type = S(from+1:end);
        end
        
        %getting name
        ind = find(strcmp(atr(:,1), name), 1);
        if ~isempty(ind)
            riskFac(end).name = atr{ind,2};
        end
        
    end
end
close(h);
%sorting
[~, ind]=sort({riskFac.name});
riskFac = riskFac(ind);

end